<?php
/******************************************************************************/
/* AEL Sarl : Web hosting, datacenter, gestion et comptabilité déportées      */
/* Copyright (C) 2008 Affaires En Ligne S.A.R.L.                              */
/* Contact : contact@affaires-en-ligne.fr                                     */
/* Web : http://www.spidboutic.fr                                             */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/*
 * Created on 14 sept. 07 by AEL Sarl for
 * 
 * osCommerce, Open Source E-Commerce Solutions
 * http://www.oscommerce.com
 *
 * Copyright (c) 2003 osCommerce
 *
 * Released under the GNU General Public License
 * 
 */

require_once("OpensiStructure.php");

class OpensiProductState {
	
	public function __construct() {
		$this->sequence = array('addManufacturer', 'addCategory');
	}
	
	public function updateProduct($opensi_product) {
		$report = null;
		foreach ($this->sequence as $method) {
			$report = $opensi_product->$method();
			if ($report)
				return $report;
		}
		return null;
	}
}

class OpensiProductUpdateState extends OpensiProductState {
	
	public function __construct() {
		parent::__construct();
		array_push($this->sequence, 'updateProduct', 'updateProductDescription', 'updateProductCategory', 'insertCorrespondence');
	}
	
}

class OpensiProductInsertState extends OpensiProductState {
	
	public function __construct() {
		parent::__construct();
		array_push($this->sequence, 'insertProduct', 'insertProductDescription', 'insertProductCategory', 'insertCorrespondence');
	}
}

class OpensiProductInfosUpdateState extends OpensiProductState {
	
	public function __construct() {
		parent::__construct();
		array_push($this->sequence, 'updateProductInfos', 'updateProductDescription', 'updateProductCategory', 'insertCorrespondence');
	}
	
}

class OpensiProductInfosInsertState extends OpensiProductState {
	
	public function __construct() {
		parent::__construct();
		array_push($this->sequence, 'insertProductInfos', 'insertProductDescription', 'insertProductCategory', 'insertCorrespondence');
	}
}

class OpensiProduct implements OpensiStructure {
	
	const REFERENCE = 0;
	const DESIGNATION = 1;
	const STOCK = 2;
	const FAMILLE = 3;
	const SOUS_FAMILLE = 4;
	const EN_LIGNE = 5;
	const DESCRIPTION = 6;
	const MARQUE = 7;
	const COLLECTION = 8;
	const POIDS = 9;
	const NOM_IMAGE = 10;
	const TVA = 11;
	const COULEUR = 12;
	const TAILLE = 13;
	const TARIF_1 = 14;
	const TARIF_2 = 15;
	const TARIF_3 = 16;
	const TARIF_4 = 17;
	const TARIF_5 = 18;
	const TARIF_1_TTC = 19;
	const TARIF_2_TTC = 20;
	const TARIF_3_TTC = 21;
	const TARIF_4_TTC = 22;
	const TARIF_5_TTC = 23;
	const PRIX_WEB = 24;
	const PRIX_ACHAT = 25;
	const ECO_TAXE = 26;
	
	/**
	 * Build a OpensiProduct object
	 * 
	 * @param	array				$product	a product, array of product parameters
	 * @param	OpensiProductState	$state		the state of the OpensiProduct object
	 */
	public function __construct($product, $state=null) {
		$this->opensi_web_id = $product[self::REFERENCE];
		$this->name = $product[self::DESIGNATION];
		$this->price = 0.0 + $product[self::TARIF_1];
		$this->quantity = 0 + $product[self::STOCK];
		$this->category = $product[self::FAMILLE];
		$this->sub_category = $product[self::SOUS_FAMILLE];
		$this->online = $product[self::EN_LIGNE];
		$this->description = $product[self::DESCRIPTION];
		$this->manufacturers_name = $product[self::MARQUE];
		$this->collection = $product[self::COLLECTION];
		$this->weight = 0.0 + $product[self::POIDS];
		$this->image_name = $product[self::NOM_IMAGE];
		$this->tax = 0.0 + $product[self::TVA];
		$this->color = $product[self::COULEUR];
		$this->size = $product[self::TAILLE];
		$this->price_1 = 0.0 + $product[self::TARIF_1];
		$this->price_2 = 0.0 + $product[self::TARIF_2];
		$this->price_3 = 0.0 + $product[self::TARIF_3];
		$this->price_4 = 0.0 + $product[self::TARIF_4];
		$this->price_5 = 0.0 + $product[self::TARIF_5];
		$this->price_1_included_tax = 0.0 + $product[self::TARIF_1_TTC];
		$this->price_2_included_tax = 0.0 + $product[self::TARIF_2_TTC];
		$this->price_3_included_tax = 0.0 + $product[self::TARIF_3_TTC];
		$this->price_4_included_tax = 0.0 + $product[self::TARIF_4_TTC];
		$this->price_5_included_tax = 0.0 + $product[self::TARIF_5_TTC];
		$this->web_price = 0.0 + $product[self::PRIX_WEB];
		$this->eco_tax = 0.0 + $product[self::ECO_TAXE];
		
		$this->product = $product;
		
		$this->products_id = 0;
		$this->categories_id = 0;
		$this->manufacturers_id = 0;
		$this->state = $state;
		
		$this->isKnown = $this->isKnownProduct();
		
	}
	
	/**
	 * check weather a product is available
	 * 
	 * @return	boolean	true if the product is available, false otherwise
	 */
	public function isAvailable() {
		$availability_query = tep_db_query("SELECT products_date_available FROM " . TABLE_PRODUCTS .
		                                   " WHERE products_id = " . $this->products_id);
		$availability = tep_db_fetch_array($availability_query);
		$products_date_available_timestamp = strtotime($availability['products_date_available']);	                                   
		
		// date_available is null or incorrect, or available in the future
		if ($products_date_available_timestamp == null ||
		    $products_date_available_timestamp > time())
			return false;
	  	
	  	return true;
	}
	
	/**
	 * Tell weather the product is known in the database
	 * 
	 * @return		boolean		the products status (true if it is considered as known)
	 * 							null if the query fails
	 */
	public function isKnownProduct() {
		$isKnown = true;
		$id_query = tep_db_query("SELECT Products_Id FROM PRODUCTS_TO_OPENSI" .
		                         " WHERE Opensi_Web_Id = '" . $this->opensi_web_id . "'");
		/*
		 * Before the first update, and the registration in the PRODUCTS_TO_OPENSI table
		 * the only known products are OsCommerce products
		 */		                         
		if ($id_query && tep_db_num_rows($id_query) > 0) {
			$product = tep_db_fetch_array($id_query);
			$this->products_id = 0 + $product['Products_Id'];
		} else { // may be the product is not registered ?
			$this->products_id = extract_product_id($this->opensi_web_id);
			$isKnown = (extract_product_id($this->opensi_web_id) != 0);
		}
		return $isKnown;
	}
	
	/**
	 * Add a manufacturer entry if needed
	 * 
	 * @return	array	a report for the update operation, 
	 *                  or null if the manufacturer was present in the database
	 */
	public function addManufacturer() {
		$this->manufacturers_id = create_manufacturer($this->manufacturers_name);
		if (tep_not_null($this->manufacturers_name) && !$this->manufacturers_id) {
			$report_msg = sprintf(PRODUCT_KO,
			                      $this->name,
			                      sprintf(CREATE_MANUFACTURER, $this->manufacturers_name));
			return array(INSERT_KO, $report_msg);
		}
		return null;
	}
	
	/**
	 * Add a couple of category/sub-categories entries if needed
	 * 
	 * @return	array	a report for the update operation, 
	 *                  or null if the couple was present in the database
	 */
	public function addCategory() {
		$this->categories_id = create_category($this->category, $this->sub_category);
		if (tep_not_null($this->category) && !$this->categories_id) {
			$report_msg = sprintf(PRODUCT_KO,
			                      $this->name,
			                      sprintf(CREATE_CATEGORIES,
			                              $this->category,
			                              $this->sub_category));
			return array(INSERT_KO, $report_msg);
		}
		return null;
	}
	
	/**
	 * Update a product description
	 * 
	 * @return	array	a report for the update operation if it fails, or null otherwise
	 */
	public function updateProductDescription() {
		$update_description_query = tep_db_query(
				                         "UPDATE " . TABLE_PRODUCTS_DESCRIPTION .
				                         " SET products_name = '" . tep_db_input($this->name) . "'," .
				                         " products_description = '" . tep_db_input($this->description) . "'" .
				                         " WHERE products_id = " . $this->products_id);
		if(tep_db_affected_rows() == -1) {
			$report_msg = sprintf(PRODUCT_KO,
			                      $this->name,
			                      UPDATE_PRODUCT_DESCRIPTION);
			return array(INSERT_KO, $report_msg);
		}
		return null;
	}

	/**
	 * Insert a new product description
	 * 
	 * @return	array	a report for the update operation if it fails, or null otherwise
	 */
	public function insertProductDescription() {
		$insert_description_query = tep_db_query(
				                         "INSERT INTO " . TABLE_PRODUCTS_DESCRIPTION .
				                         " (products_id, language_id, products_name, products_description) VALUES(" .
				                         $this->products_id . ", " . OPENSI_LANGUAGE . ", '"  . 
				                         tep_db_input($this->name) . "', '" . tep_db_input($this->description) . "')");
		if(tep_db_affected_rows() <= 0) {
			$report_msg = sprintf(PRODUCT_KO,
			                      $this->name,
			                      CREATE_PRODUCT_DESCRIPTION);
			return array(INSERT_KO, $report_msg);
		}
		return null;
	}

	/**
	 * Update the correspondence between a product and its category
	 * 
	 * @return	array	a report for the update operation if it fails, or null otherwise
	 */
	public function updateProductCategory() {
		if(!$this->categories_id)
			tep_db_query("DELETE FROM " . TABLE_PRODUCTS_TO_CATEGORIES .
			             " WHERE products_id = " . $this->products_id);
	    else {
	    	/*
	    	* We choose the INSERT...ON DUPLICATE KEY UPDATE syntax cause
	 		* a product may have been previously inserted with an empty category.
	 		* In this latter case, there isn't any corresponding value for the products_id
	 		* in the products_to_categories table. The update operation will fail silently.
	 		*/
			$update_product_category_query = tep_db_query(
					                	         "INSERT INTO " . TABLE_PRODUCTS_TO_CATEGORIES .
					                	         " (products_id, categories_id) VALUES(" .
					                       		 $this->products_id . ", " . $this->categories_id . ")" .
					                       		 " ON DUPLICATE KEY UPDATE" .
					                       		 " products_id=VALUES(products_id), categories_id=VALUES(categories_id)");
			if(tep_db_affected_rows() == -1) {
				$report_msg = sprintf(PRODUCT_KO,
				                      $this->name,
				                      UPDATE_PRODUCT_CATEGORY);
				return array(INSERT_KO, $report_msg);
			}
	    }
		return null;
	}
	
	/**
	 * Create the correspondence between a product and its category
	 * 
	 * @return	array	a report for the update operation if it fails, or null otherwise
	 */
	public function insertProductCategory() {
		if($this->categories_id) {
			$insert_product_category_query = tep_db_query(
					                	         "INSERT INTO " . TABLE_PRODUCTS_TO_CATEGORIES .
					                       		 " (products_id, categories_id) VALUES(" .
												 $this->products_id . ", " . $this->categories_id . ")"); 
			if(tep_db_affected_rows() <= 0) {
				$report_msg = sprintf(PRODUCT_KO,
				                      $this->name,
				                      CREATE_PRODUCT_CATEGORY);
				return array(INSERT_KO, $report_msg);
			}
		}
		return null;
	}
	
	/**
	 * Update an existing product
	 * 
	 * @return	array	a report for the update operation if it fails, or null otherwise
	 */
	public function updateProduct() {
		$update_product_query_string = "UPDATE " . TABLE_PRODUCTS .
									   " SET products_price = " . $this->price . "," .
									   " products_last_modified = NOW()," .
									   " manufacturers_id = " . $this->manufacturers_id . "," .
									   " products_weight = " . $this->weight;
		$tax_class_id = opensi_get_tax_class_id($this->tax);
		$qty = $this->quantity;
		if ($tax_class_id > 0)
			$update_product_query_string .= ", products_tax_class_id = " . $tax_class_id;
		if ($this->online && !$this->isAvailable()) // toggle the availability from unavailable to available
			$update_product_query_string .= ", products_date_available = NOW()";
		else if(!$this->online) {
			$update_product_query_string .= ", products_date_available = NULL";								   
			$qty = 0;
		}
		$update_product_query_string .= ", products_quantity = " . $qty;
		$update_product_query_string .= " WHERE products_id = " . $this->products_id;
		$update_product_query = tep_db_query($update_product_query_string);
		if (tep_db_affected_rows() == -1)	{
			$report_msg = sprintf(PRODUCT_KO, $this->name, UPDATE_PRODUCT);
			return array(INSERT_KO, $report_msg);
		}
		return null;
	}
	
	/**
	 * Insert a new product
	 * 
	 * @return	array	a report for the insert operation if it fails, or null otherwise
	 */
	public function insertProduct() {
		$insert_product_query = tep_db_query(
									"INSERT INTO " . TABLE_PRODUCTS .
									" (products_quantity, products_image, products_price, products_date_added," .
									" products_weight, products_tax_class_id, manufacturers_id, products_date_available) VALUES (" .
									$this->quantity . ", '', " . $this->price .	", NOW(), " .
									$this->weight . ", " . opensi_get_tax_class_id($this->tax) . ", " . 
									$this->manufacturers_id . ", " . ($this->online ? "NOW()" : "NULL") . ")"); 
		$this->products_id = tep_db_insert_id();
								
		if (tep_db_affected_rows() <= 0)	{
			$report_msg = sprintf(PRODUCT_KO, $this->name, CREATE_PRODUCT);
			return array(INSERT_KO, $report_msg);
		}
		return null;
	}
	
	/**
	 * Update products infos fields
	 * 
	 * @return	array	a report for the insert operation if it fails, or null otherwise
	 */
	public function updateProductInfos() {
		$update_infos_query = tep_db_query(
								"UPDATE " . TABLE_PRODUCTS .
								" SET products_last_modified = NOW()," .
								" manufacturers_id = " . $this->manufacturers_id . "," .
								" products_weight = " . $this->weight .
								" WHERE products_id = " . $this->products_id);
		if (tep_db_affected_rows() == -1)	{
			$report_msg = sprintf(PRODUCT_KO, $this->name, UPDATE_INFO_PRODUCT);
			return array(INSERT_KO, $report_msg);
		}
		return null;
	}
	
	/**
	 * Insert a new product, but with quantity and price to default values. Beware.
	 * 
	 * @return	array	a report for the insert operation if it fails, or null otherwise
	 */
	public function insertProductInfos() {
		$insert_infos_query = tep_db_query(
								"INSERT INTO " . TABLE_PRODUCTS .
								" (products_model, products_image, products_date_added," .
								" products_weight, products_tax_class_id, manufacturers_id) VALUES (" .
								"'" . tep_db_input($this->name) . "', '', NOW(), " .
								$this->weight . ", 4, " . $this->manufacturers_id . ")"); 
		$this->products_id = tep_db_insert_id();
								
		if (tep_db_affected_rows() <= 0)	{
			$report_msg = sprintf(PRODUCT_KO, $this->name, CREATE_INFO_PRODUCT);
			return array(INSERT_KO, $report_msg);
		}
		return null;
	}
	
	/**
	 * Update products price fields
	 * 
	 * @return	array	a report for the update operation
	 */
	public function updateProductPrice() {
		$update_price_query = tep_db_query(
		                         "UPDATE " . TABLE_PRODUCTS .
		                         " SET products_price = " . $this->price .
		                         " WHERE products_id = " . $this->products_id);
		return (tep_db_affected_rows() >= 0 ? 
		        	array(UPDATE_OK, 'article ' . $this->opensi_web_id) :
		        	array(UPDATE_KO, 'article ' . $this->opensi_web_id));		                         
	}
	
	
	/**
	 * Insert a correspondence between an OsCommerce product and the OpenSI product
	 */
	public function insertCorrespondence() {
		$insert_correspondence_query = tep_db_query(
									"INSERT INTO PRODUCTS_TO_OPENSI (Products_Id, Opensi_Web_Id) VALUES(" .
									$this->products_id . ", '" . $this->opensi_web_id . "')" .
									" ON DUPLICATE KEY UPDATE" .
									" Products_Id=VALUES(Products_Id), Opensi_Web_Id=VALUES(Opensi_Web_Id)");
		if (tep_db_affected_rows() == -1)	{
			$report_msg = sprintf(PRODUCT_KO, $this->name, CREATE_PRODUCT_TO_OPENSI);
			return array(INSERT_KO, $report_msg);
		}
		return null;
	}
	
	/**
	 * Perform an operation on a product: 
	 * insert a new product or update an existing one
	 */
	public function updateOscProduct() {
		if($this->state)
			return $this->state->updateProduct($this);
		return null;
	}
	
	/**
	 * Prepare a product for an upload toward the OpenSI server
	 */
	public function prepareUpload() {
		
		$last_backslash = strrpos($this->image_name, "/");
		$basename_image = ($last_backslash === false ? 
							   $this->image_name :
							   substr(strrchr($this->image_name, "/"), 1));
			
    	// return a modified copy
		$prepared = $this->product;
		$prepared[self::NOM_IMAGE] = $basename_image;
		// manufacturer_name is required in OpenSI (in the Save function)
		$prepared[self::MARQUE] = (tep_not_null($this->manufacturers_name) ?
		                           $this->manufacturers_name :
		                           opensi_soap_decode(UNKNOWN_MANUFACTURER));
    	return $prepared;
	}

	
}
?>
